#!/bin/bash
# shellcheck disable=SC2034 # Variables declared this file are used elsewhere

# localization
TEXTDOMAIN=scripts
TEXTDOMAINDIR="${PROJECT_HOME:-.}/scripts/locale"

INFO_FILE="${PROJECT_HOME:-.}/info.json"

error() {
	printf "%s\n" "$@" >&2
	exit 1
}

get_arch() {
	ARCH=$(jq -r ".arch // empty" "${INFO_FILE}")
	[ -n "$ARCH" ] ||
		error $"Could not read arch from info.json" \
			$"info.json should have been created at project creation, please recreate" \
			$"the project from ABOSDE and contact support if this does not help."
}

get_carch() {
	[ -n "$ARCH" ] || get_arch
	case "$ARCH" in
	arm64 | aarch64) CARCH="arm64v8" ;;
	armv7) CARCH="arm32v7" ;;
	armhf) CARCH="arm32v6" ;;
	x86_64) CARCH="amd64" ;;
	*) error $"Got invalid arch: $ARCH" ;;
	esac
}

get_product() {
	PRODUCT=$(jq -r ".product // empty" "${INFO_FILE}")
	[ -n "$PRODUCT" ] ||
		error $"Could not read product from info.json" \
			$"info.json should have been created at project creation, please recreate" \
			$"the project from ABOSDE and contact support if this does not help."
}

get_version() {
	VERSION=$(jq -r ".version // empty" "${INFO_FILE}")
	[ -n "$VERSION" ] || VERSION="0"
}

get_project_type() {
	PROJECT_TYPE=$(jq -r ".type // empty" "${INFO_FILE}")
	[ -n "$PROJECT_TYPE" ] ||
		error $"Could not read type from info.json" \
			$"after updating ABOSDE to 1.5.0 or later, please recreate" \
			$"the project from ABOSDE and contact support if this does not help."
}

get_project_name() {
	PROJECT=$(basename "$PWD")
	PROJECT="${PROJECT,,}"

	[[ "$PROJECT" == "a6e-gw-container" ]] &&
		error $"Do not use a6e-gw-container in this project name, because gw container image uses a6e-gw-container. Please change the project name."
}

prompt_yesno() {
	local default="$1"
	shift 1
	local yesno

	while true; do
		read -r -p "$1" yesno
		[ -n "$yesno" ] || yesno="$default"
		case "$yesno" in
		[Yy] | [Yy][Ee][Ss] | [Tt][Rr][Uu][Ee] | 1)
			return 0
			;;
		[Nn] | [Nn][Oo] | [Ff][Aa][Ll][Ss][Ee] | 0)
			return 1
			;;
		esac
	done
}

kill_app() {
	get_project_name
	get_project_type
	container="$PROJECT"

	[[ "$PROJECT_TYPE" == "gateway" ]] &&
		container="a6e-gw-container"

	ssh -F config/ssh_config Armadillo "podman kill ${container}"
}

copy_run() {
	local source dest container

	[ "$#" -lt 2 ] && return
	source=$1
	dest=$2
	shift 2

	get_project_type
	if [[ "$PROJECT_TYPE" == "gateway" ]]; then
		container_exist=$(ssh -F config/ssh_config Armadillo "podman images a6e-gw-container | grep a6e-gw-container")
		if [ -z "$container_exist" ]; then
			error $"This gw project failed to start because gw container image does not exist." \
				$"Please gw container image download and install from https://armadillo.atmark-techno.com/armadillo-iot-a6e/resources/software"
		fi
	fi

	if [[ -d "$source" ]]; then
		# copy to dest without removing dest dir (breaks volume)
		tar -C "$source" -cz . |
			ssh -F config/ssh_config Armadillo '
				d='"${dest@Q}"';
				find "$d" -mindepth 1 -delete && tar -C "$d" -xz'
	else
		scp -F config/ssh_config "$source" Armadillo:"$dest"
		# make config file changes persistent
		ssh -F config/ssh_config Armadillo "persist_file $dest"
	fi

	# Copy only -- stop here.
	[ "$#" = 0 ] && return
	container="$1"

	ssh -F config/ssh_config Armadillo "podman_start $container \
		&& podman attach $container"
}

confirm_crlf() {
	dir=$1

	# Confirm that there is no newline symbol in Windows
	grep -qr $'\r' --include={*.sh,*.py} "$dir" &&
		grep -lr $'\r' --include={*.sh,*.py} "$dir" &&
		error $"linefeed symbol in the above file is CRLF. Please change it to LF"
}

build_project() {
	PROJECT_HOME="$(realpath -s --relative-to . "$(dirname "$0")/..")" || exit
	get_project_name
	get_carch
	get_product
	IMAGE="${CARCH}/${PROJECT}"
	CHECK_IMAGE_NAME="localhost/${IMAGE}_build"

	podman image exists "$CHECK_IMAGE_NAME" || error $"Container image for build does not exist."

	# Create executable files, etc.
	mkdir -p "$PROJECT_HOME/app/build" "$PROJECT_HOME/app/lib"
	podman run --rm \
			   -v ./app:/vol_app \
			   "${CHECK_IMAGE_NAME}" \
			   /bin/sh -c "make -C /vol_app/src"

	# Copy executable files to /swu/dest directory.
	TRANSFER_FILES_DIR="${PROJECT_HOME}/swu/dest"
	rm -rf "$TRANSFER_FILES_DIR"
	mkdir -p "$TRANSFER_FILES_DIR"
	cp -a "${PROJECT_HOME}/app/build" "${PROJECT_HOME}/app/lib" "$TRANSFER_FILES_DIR"
}
